﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
//using Microsoft.VisualBasic;
//using System.Collections;
using System.Diagnostics;

namespace SocketClient
{
    public partial class SClient : Form
    {
        public SClient()
        {
            InitializeComponent();
        }

	    private System.Net.Sockets.NetworkStream NetStream; //Network Stream object
	    private System.Net.Sockets.TcpClient TcpClient;     //TcpClient object

	    /// <summary>
	    /// Sends message to EI
	    /// </summary>
	    /// <param name="sender"></param>
	    /// <param name="e"></param>
	    /// <remarks></remarks>
	    private void btnSend_Click(System.Object sender, System.EventArgs e)
	    {
		    string ip = null;
		    //IP address
		    int pn = 0;
		    //Port number
		    string cmd = null;
		    //Command
		    string response = null;
		    //Response from EasyInspector

		    try {
			    ip = this.txtIPAddr.Text;
			    pn = Convert.ToInt32(this.txtPort.Text);
			    cmd = this.txtCommand.Text;

			    response = Communicate(ip, pn, cmd, 0, true);
			    this.lstResp.Items.Insert(0, response);

		    } catch (Exception ex) {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

		    }
	    }

	    /// <summary>
	    /// Sends a command to EI and receives its response.
	    /// </summary>
	    /// <param name="ipaddr">IP address</param>
	    /// <param name="portno">Port number</param>
	    /// <param name="sendstr">Command string</param>
	    /// <param name="retrymax">Max. number of retries</param>
	    /// <param name="showerror">True: Shows error dialog if communication failed.</param>
	    /// <returns>Response from EI</returns>
	    /// <remarks></remarks>
	    public string Communicate(string ipaddr, int portno, string sendstr, int retrymax, bool showerror)
	    {
		    int rtry = 0;
		    //Retry counter
		    string response = null;
		    //Response from EasyInspector

		    do {
			    try {
				    response = TcpSendRecv(ipaddr, portno, sendstr);

				    if (!string.IsNullOrEmpty(response)) {
					    return response;
				    } else {
					    rtry += 1;
					    if (rtry > retrymax) {
						    if (showerror == true) {
                                MessageBox.Show( "Communication failed. Check the communication settings of EasyInspector. \r\nIP address = " + ipaddr 
                                                        + "\r\nPort No. = " + portno
                                                        , "Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
						    }
						    return "";

					    }
					    System.Threading.Thread.Sleep(500);
				    }

			    } catch (Exception ex) {
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			    }
		    } while (true);

	    }

        /// <summary>
        /// Sends the "command" to ther server specified in "ipaddr" and "port", and returns the response from the server.
        /// </summary>
        /// <param name="ipaddr">Target IP address</param>
        /// <param name="port">Target Port number</param>
        /// <param name="command">Command to send</param>
        /// <returns>Response from server</returns>
        /// <remarks></remarks>
        internal string TcpSendRecv(string ipaddr, int port, string command)
        {
            byte[] RBuff = new byte[65536];//Receive Buffer
            byte[] SBuff = null;//Send Buffer
            int StrLen = 0;//Lengh of string
            string RStr = null;//Received String

            TcpClient = new System.Net.Sockets.TcpClient();
            try
            {
                TcpClient.Connect(ipaddr, port);
                NetStream = TcpClient.GetStream();

                SBuff = System.Text.Encoding.UTF8.GetBytes(command);
                NetStream.Write(SBuff, 0, SBuff.Length);
                do
                {
                    System.Threading.Thread.Sleep(10);

                    if (NetStream.DataAvailable)
                    {
                        StrLen = NetStream.Read(RBuff, 0, RBuff.Length);
                        RStr += System.Text.Encoding.UTF8.GetString(RBuff).Substring(0, StrLen);
                    }
                    else if ((RStr != null))
                    {
                        break; // TODO: might not be correct. Was : Exit Do
                    }
                } while (true);

            }
            catch (Exception Ex)
            {
                RStr = "";
                MessageBox.Show(Ex.Message);
            }

            TcpClient.Close();
            return RStr;

        }
    }
}
